// Copyright 2006-207 Pierre Evenou
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// pierre.evenou@univ-nantes.fr

#include "Angle.hpp"

namespace Mojette
{

Angle::Angle(short p, unsigned short q) : mP(p), mQ(q)
{
    reduce();
}

bool
Angle::operator ==(const Angle& a)
{
    return mP == a.mP && mQ == a.mQ;
}
 
bool
Angle::operator !=(const Angle& a)
{
    return mP != a.mP || mQ != a.mQ;
}
 
bool
Angle::operator >(const Angle& a)
{
    return this->getTheta() > a.getTheta();
}
 
bool
Angle::operator <(const Angle& a)
{
    return this->getTheta() < a.getTheta();
}

void
Angle::reduce()
{
    short n;
    unsigned short d;
    int t;

    if (mQ == 0)
    {
        mP = 1;
    }
    else if (mP == 0)
    {
        mQ = 1;
    }
    else
    {
        n = mP < 0 ? -mP : mP;
        d = mQ;

        // Euclide.
        while (d)
        {
            t = d;
            d = n % t;
            n = t;
        }

        mP /= n;
        mQ /= n;
    }
}

bool
SortByAngle::operator()(const Angle& a1, const Angle& a2) const
{
    return a1.getTheta() < a2.getTheta();
}

}
