// Copyright 2006 Pierre Evenou
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// pierre.evenou@univ-nantes.fr

#include "Rational.hpp"

namespace Mojette
{
Rational::Rational() : mNum(0), mDen(1)
{}

Rational::Rational(int num) : mNum(num), mDen(1)
{}

Rational::Rational(int num, int den) : mNum(num), mDen(den)
{}

Rational
Rational::operator +(const Rational& r) const
{
    return Rational(mNum * r.mDen + r.mNum * mDen, mDen * r.mDen);
}

Rational&
Rational::operator +=(const Rational& r)
{
    (*this) = (*this) + r;

    return (*this);
}

Rational
Rational::operator -(const Rational& r) const
{
    return Rational(- (mNum * r.mDen + r.mNum * mDen),
                    mDen * r.mDen);
}

Rational&
Rational::operator -=(const Rational& r)
{
    (*this) = (*this) - r;

    return (*this);
}

Rational
Rational::operator *(const Rational& r) const
{
    return Rational(mNum * r.mNum, mDen * r.mDen);
}

Rational&
Rational::operator *=(const Rational& r)
{
    (*this) = (*this) * r;

    return (*this);
}

Rational
Rational::operator /(const Rational& r) const
{
    Rational tmp(r.mDen, r.mNum);
    return (*this) * tmp;
}

Rational&
Rational::operator /=(const Rational& r)
{
    (*this) = (*this) / r;

    return (*this);
}

Rational::operator double()
{
    return (double)mNum / (double)mDen;
}

void
Rational::reduce()
{
    int n;
    int d;
    int t;

    n = mNum < 0 ? -mNum : mNum;
    d = mDen < 0 ? -mDen : mDen;

    // Euclide.
    while (d)
    {
        t = d;
        d = n % t;
        n = t;
    }

    mNum /= n;
    mDen /= n;

    if (mDen < 0)
    {
        mNum = -mNum;
        mDen = -mDen;
    }
}

int
Rational::getNum() const
{
    return mNum;
}

int
Rational::getDen() const
{
    return mDen;
}

std::ostream &
operator<<(std::ostream &ostrm, const Rational& r)
{
    ostrm << r.getNum() << "/" << r.getDen();
    return ostrm;
}

}

