#include <iostream>
#include <fstream>
#include <iomanip>
#include <iterator>
#include <vector>
#include <sys/resource.h>

#include "Mojette.hpp"

int
main( int argc, char **argv)
{
    struct rusage usage;
    float begin;
    float end;

	std::cout << "START" << std::endl;
	
    Mojette::DiracTransform<char, char, Mojette::XOR> dt(512, 512);
    
    std::vector<char> pixels;
    std::ifstream in("./lena.raw", std::ios::binary);
    for (int i = 0; i < 512*512; i ++)
    {
        char c;
        in.read(&c, sizeof(char));
        pixels.push_back(c);
    }
    //std::copy(std::istream_iterator<char>(in), std::istream_iterator<char>(), std::back_inserter(pixels));
    in.close();
    
    std::vector<Mojette::Angle> angles;
    angles.push_back(Mojette::Angle(1, 256));
    angles.push_back(Mojette::Angle(1, 257));
    angles.push_back(Mojette::Angle(1, 258));
    
    std::cout << "FORWARD : ";
    
    getrusage(RUSAGE_SELF, &usage);
    begin = (float)usage.ru_utime.tv_sec + (float)usage.ru_stime.tv_sec + (float)((float)usage.ru_utime.tv_usec + (float)usage.ru_stime.tv_usec) / (float)1000000;
    
    std::vector<std::vector<char> >* p_set = dt.forward(angles, pixels);
    
    getrusage(RUSAGE_SELF, &usage);
    end = (float)usage.ru_utime.tv_sec + (float)usage.ru_stime.tv_sec + (float)((float)usage.ru_utime.tv_usec + (float)usage.ru_stime.tv_usec) / (float)1000000;
    
    std::cout << end - begin << std::endl;
    
    assert(! p_set->empty());
    
    std::cout << "INVERSE : ";
    
    std::vector<Mojette::Angle> angles_rec;
    angles_rec.push_back(Mojette::Angle(1, 256));
    angles_rec.push_back(Mojette::Angle(1, 257));
        
    std::vector<std::vector<char> > p_set_rec;
    p_set_rec.push_back(p_set->operator[](1));
    p_set_rec.push_back(p_set->operator[](2));
    
    getrusage(RUSAGE_SELF, &usage);
    begin = (float)usage.ru_utime.tv_sec + (float)usage.ru_stime.tv_sec + (float)((float)usage.ru_utime.tv_usec + (float)usage.ru_stime.tv_usec) / (float)1000000;
    
    std::vector<char>* pixels_rec = dt.inverse(angles_rec, p_set_rec);

    getrusage(RUSAGE_SELF, &usage);
    end = (float)usage.ru_utime.tv_sec + (float)usage.ru_stime.tv_sec + (float)((float)usage.ru_utime.tv_usec + (float)usage.ru_stime.tv_usec) / (float)1000000;
    
    std::cout << end - begin << std::endl;
    
    assert(! pixels_rec->empty());
    
    std::ofstream out("./lena_rec.raw");
    for (int i = 0; i < 512*512; i ++)
    {
        out.write(&pixels[i], sizeof(char));
    }
    //std::copy(pixels_rec->begin(),pixels_rec->end(),std::ostream_iterator<char>(out));
    out.close();
    
    return 0;
}